function [m,Freq,Amp,L] = SMI_harmonic(SMI_signal,Samp_Rate,L0,lambda) 
%% Pablo Garca - UQ 2017
% INPUT: 
% SMI_signal = Signal from the SMI (column vector)
% Samp_Rate = Sampling period (samples/time) 
% L0 = Rest position of the target (m) and different than 0
% lambda =  Wavelength of the laser (in meters)
% OUTPUT:
% m = Total number of fringes
% Freq = Frequency of the motion
% Amp = Amplitude of the motion (in meters)
% L = Reconstructed motion (in meters)


%% Set parameters
pos_error = 3E-5*Samp_Rate; % Set a minimum distance between peaks 
threshold = 0.4; % Set a threshold for a peak
DeltaX = lambda/2; % In meters
h_der = 1/Samp_Rate; % Time step difference

%% Filter data 
Window_Size = round(pos_error); % round(N_samples/300); % Create the filtering window
b = (1/Window_Size)*ones(1,Window_Size); a = [1]; % Moving mean filter 
SMI_filter = filter(b,a,SMI_signal); 

figure(1)
plot(SMI_signal,'-r'); hold on; plot(SMI_filter,'-b');

%% Differenciate the signal and filter the 2nd derviative
% 1st FW der from the filtered signal; BC: S_derv(t=T) = S_derv(t=T-h)
S_derv = (1/h_der)*(SMI_filter(2:end)-SMI_filter(1:end-1)); 
S_derv = [S_derv; S_derv(end)];

% 2nd FW derv; BC: S_derv2(t=T) = S_derv2(t=T-h) = S_derv'(t=T-h)
S_derv2 = (1/h_der^2)*(SMI_filter(3:end)-2*SMI_filter(2:end-1)...
    +SMI_filter(1:end-2)); 
S_derv2 = [S_derv2; (1/h_der)*(S_derv(end)-S_derv(end-1))];
S_derv2 = [S_derv2; S_derv2(end)]; 
% Clear any possible super-high value
S_derv2(S_derv2>1E3) = 0; S_derv2_filter = filter(b,a,S_derv2);

figure(2); 
subplot(3,1,1); plot(SMI_filter); hold on; plot(SMI_signal,'--');hold off;
subplot(3,1,2); plot(S_derv); 
subplot(3,1,3); plot(S_derv2);

%% Normalize the signal and find the transitions
S_derv2_norm = S_derv2_filter/max(abs(S_derv2_filter)); % Normalize the SMI signal
pos = find(abs(S_derv2_norm)>threshold); % Find all values position above the threshold

figure(3)
plot(S_derv2_norm,'--r'); hold on; 
line([0 length(S_derv2_norm)],[threshold threshold]); 
line([0 length(S_derv2_norm)],[-threshold -threshold]); 
hold off;

if isempty(pos) == 0 % Check if there is any found peak
    m = 1; last_pos = pos(1); % Initialize the counting and save the position of the peak
    Dir = -sign(SMI_filter(pos(1)));
    
    for jj = 2:length(pos)
        if pos(jj) - last_pos > pos_error % Check if they are not continuos points
            if pos(jj) - last_pos > 3*pos_error; Dir = -Dir; end
            trans(pos(jj)) = Dir;
            m = m + 1; last_pos = pos(jj); % Sum one peak and actualize the last position
            figure(4)
            plot(trans,'-b'); hold on; plot(SMI_signal/max(abs(SMI_signal)),'--r');
        end
    end
    trans(pos(end)) = -sign(S_derv(pos(end)));
else
    m = 0;
end

%% Reconstruct the signal
L(1) = 0; jj = 2;
while jj < length(trans)
    L(jj) = L(jj-1) + DeltaX*trans(jj); 
    jj = jj + 1;
end
ind = find(L(2:end).*L(1:end-1)<0,2);
Offset_L = L0 - L(ind(1));
Freq = 1/(2*h_der*(ind(2)-ind(1))); Amp = max(L)-min(L);
L = L + Offset_L;

%% Fit: 'Harmonic Motion'.
% L = L0 + A*cos(wt + phi) = L0 + A*cos(2*pi*f*t + phi)
[xData, yData] = prepareCurveData( [], L );

% Set up fittype and options.
ft = fittype( 'L0 + a*cos(w*x+c)', 'independent', 'x', 'dependent', 'y' );
opts = fitoptions( 'Method', 'NonlinearLeastSquares' );
opts.Display = 'Off';
opts.Lower = [L0 0 -pi/2 0];
opts.StartPoint = [L0 Amp/2 acos(L(1)) 2*pi*1E3/Samp_Rate]; % 2*pi*1E3/Samp_Rate
opts.Upper = [L0 Amp/2 pi/2 Freq];

% Fit model to data.
[fitresult, gof] = fit( xData, yData, ft, opts );
fit_values = coeffvalues(fitresult); 
Amp = 2*fit_values(2); Freq = fit_values(4)/(2*pi)*Samp_Rate;

% Plot fit with data.
figure(5);
subplot(211)
plot(linspace(0,length(SMI_signal)/Samp_Rate,length(SMI_signal)), ...
    SMI_signal/max(abs(SMI_signal)));
xlabel('Time (s)')
ylabel('Power variation (a.u.)')

subplot(212)
h = plot( fitresult, xData, yData );
legend( h, 'Fringes signal', 'Harmonic Motion Reconstruction', 'Location', 'NorthEast' );
xlabel('Time units')
ylabel 'L (m)'
grid on

end
