function [f_Doppler, RPM, f_2nd, C, Fit_PSD] = spinning_disk(lambda,n_ext,theta,radius,PSD,Freq_vec)
%% Pablo Garcia - UQ 2017
% INPUT parameters:
% lambda = Wavelength of the laser (nm)
% n_ext = Refraction index (default is air n_ext = 1)
% theta = Angle with respect to the optical axis ()
% radius = Distance from the center of the laser spot (cm)
% PSD = PSD of the signal (dB)
% Freq_vec = Frequencies of the FFT
% OUTPUT parameters:
% f_Doppler = Estimated Doppler frequency (Hz)
% RPM = rev/min of the spinning disk
% f_2nd = Estimated 2nd hamrmonic frequency (Hz)
% C = Estimated optical feedback parameter
% Fit_PSD = PSD fitting
close all

%% Filter Data
F_Max = Freq_vec(end); dF = Freq_vec(2)-Freq_vec(1);

N_freqs = length(Freq_vec);
Window_Size = round(N_freqs/1E2); % Create the filtering window
b = (1/Window_Size)*ones(1,Window_Size); a = [1]; % Moving mean filter 
filter_PSD = filter(b,a,PSD); 
figure('Name','Filter')
plot(Freq_vec,PSD,'ob')
hold on
plot(Freq_vec(Window_Size/2:end-Window_Size/2),filter_PSD(Window_Size:end),'*r')
hold off

Freq_vec = Freq_vec(Window_Size/2:end-Window_Size/2);
filter_PSD = filter_PSD(Window_Size:end);

%% Fit Background
% Ignore the first 1kHz 
[xData, yData] = prepareCurveData(Freq_vec,filter_PSD);

% Set up fittype and options.
ft = fittype( 'exp2' );
opts = fitoptions( 'Method', 'NonlinearLeastSquares' );
opts.StartPoint = [-50 5.57570942425215e-07 15.8451405860655 -1.3684194531878e-05];
excludePoints = excludedata( xData, yData, 'Indices',(1:1E3));
opts.Exclude = excludePoints;

% Fit model to data.
[Background_res, gof] = fit( xData, yData, ft, opts );

% Plot fit with data.
figure( 'Name', 'Background_Fitting' );
h = plot( Background_res, xData, yData);
legend( h, 'Filtered PSD', 'Background Fitting', 'Location', 'NorthEast' );
xlabel Freq(Hz)
ylabel PSD(dB)
grid on

%% Remove Background and fit the Gaussian peak
Back_fit_values = coeffvalues(Background_res); 
Background = Back_fit_values(1)*exp(Back_fit_values(2)*Freq_vec)+...
    Back_fit_values(3)*exp(Back_fit_values(4)*Freq_vec);
rem_PSD = filter_PSD - Background;

[xData, yData] = prepareCurveData(Freq_vec,rem_PSD);

% Set up fittype and options.
ft = fittype( 'gauss1' );
opts = fitoptions( 'Method', 'NonlinearLeastSquares' );
opts.Display = 'Off';
opts.Lower = [0 1E3 0];
ind = find((rem_PSD(2:end) - max(rem_PSD)/2).*...
    (rem_PSD(1:end-1) - max(rem_PSD)/2)<0,2);
% A; f_Doppler; f_FWHM
opts.StartPoint = [max(rem_PSD)/2 Freq_vec(ind(1)) ...
    Freq_vec(ind(2))-Freq_vec(ind(1))]; 
excludePoints = excludedata( xData, yData, 'Indices',(1:1E3));
opts.Exclude = excludePoints;

% Fit model to data.
[Gaussian_Peak, gof] = fit( xData, yData, ft, opts );

% Plot fit with data.
figure( 'Name', 'Gaussian Peak fitting' );
h = plot( Gaussian_Peak, xData, yData );
legend( h, 'Filtered PSD - Background', 'Gaussian Peak fitting', 'Location', 'NorthEast' );
xlabel Freq(Hz)
grid on

Gaussian_values = coeffvalues(Gaussian_Peak); 
Amp_Doppler = Gaussian_values(1);
f_Doppler = Gaussian_values(2); f_FWHM = Gaussian_values(3);

%% Find the 2nd harmonic
f_lower = find(Freq_vec > f_Doppler + 2*f_FWHM, 1);
ind = find(((rem_PSD(f_lower+1:end)).*(rem_PSD(f_lower:end-1)))<0,2);

[xData, yData] = prepareCurveData(Freq_vec(f_lower:end),filter_PSD(f_lower:end));

% Set up fittype and options.
ft = fittype( 'exp2' );
opts = fitoptions( 'Method', 'NonlinearLeastSquares' );
opts.Display = 'Off'; 

% Fit model to data.
[Background_2H, gof] = fit( xData, yData, ft, opts );

% Plot fit with data.
figure( 'Name', 'Background removal for 2nd Harmonic' );
h = plot( Background_2H, xData, yData );
legend( h, 'Filtered PSD 2nd H window', '2nd H Background', 'Location', 'NorthEast' );
xlabel Freq(Hz)
grid on

%% Remove Background and find the 2nd Harmonic
Background_2H_values = coeffvalues(Background_2H); 
Background = Background_2H_values(1)*exp(Background_2H_values(2)*Freq_vec(f_lower:end))+...
    Background_2H_values(3)*exp(Background_2H_values(4)*Freq_vec(f_lower:end));
rem_PSD = filter_PSD(f_lower:end) - Background;
Freq_window = Freq_vec(f_lower:end);

ind = []; jj = 2;
while isempty(ind) == 1 
    ind = find((rem_PSD(2:end) - max(rem_PSD)/jj).*...
    (rem_PSD(1:end-1) - max(rem_PSD)/jj)<0,2);
    jj = jj + 1;
end

i_min = ind(1) - round((ind(1)+ind(2))/2); 
if i_min<0; i_min = 1; end
i_max = ind(2) + round((ind(1)+ind(2))/2);
if i_max > length(rem_PSD); i_max = length(rem_PSD) - 1; end

[xData, yData] = prepareCurveData(Freq_window(i_min:i_max+1),rem_PSD(i_min:i_max+1));

% Set up fittype and options.
ft = fittype( 'gauss1' );
opts = fitoptions( 'Method', 'NonlinearLeastSquares' );
opts.Display = 'Off';
opts.Lower = [0 1E3 0];
% A; f_Doppler; f_FWHM
opts.StartPoint = [max(rem_PSD)/2 Freq_window(ind(1)) ...
    Freq_window(ind(2))-Freq_window(ind(1))]; 

% Fit model to data.
[Gaussian_Peak, gof] = fit( xData, yData, ft, opts );

% Plot fit with data.
figure( 'Name', 'Gaussian Peak 2H fitting' );
h = plot( Gaussian_Peak, xData, yData );
legend( h, 'Filtered PSD - Background', 'Gaussian Peak fitting', 'Location', 'NorthEast' );
% Label axes
xlabel Freq(Hz)
grid on

Gaussian_2nd_values = coeffvalues(Gaussian_Peak); 
Amp_2nd = Gaussian_2nd_values(1);
f_2nd = Gaussian_2nd_values(2); f_FWHM_2nd = Gaussian_2nd_values(3);

%% Fit Background without Peaks
% Ignore the first 1kHz 
[xData, yData] = prepareCurveData(Freq_vec,filter_PSD);

% Set up fittype and options.
ft = fittype( 'exp2' );
opts = fitoptions( 'Method', 'NonlinearLeastSquares' );
opts.StartPoint = [-50 5.57570942425215e-07 15.8451405860655 -1.3684194531878e-05];
excludePoints = excludedata( xData, yData, 'Indices',...
    [(1:1E3) ...
    (find(Freq_vec<f_Doppler-2*f_FWHM,1,'last'):find(Freq_vec<f_Doppler+2*f_FWHM,1))...
     (find(Freq_vec<f_2nd-2*f_FWHM_2nd,1,'last'):find(Freq_vec<f_2nd+2*f_FWHM_2nd,1))]);
opts.Exclude = excludePoints;

% Fit model to data.
[Background_res, gof] = fit( xData, yData, ft, opts );

% Plot fit with data.
figure( 'Name', 'Background_Fitting' );
h = plot( Background_res, xData, yData);
legend( h, 'Filtered PSD', 'Background Fitting', 'Location', 'NorthEast' );
xlabel Freq(Hz)
ylabel PSD(dB)
grid on

B_coefs = coeffvalues(Background_res);

%% Results and final plot
Fit_PSD = B_coefs(1)*exp(B_coefs(2)*(0:dF:F_Max)) + ...
    B_coefs(3)*exp(B_coefs(4)*(0:dF:F_Max)) + ...
    Amp_Doppler*exp(-(((0:dF:F_Max)-f_Doppler)/f_FWHM).^2) +...
    Amp_2nd*exp(-(((0:dF:F_Max)-f_2nd)/f_FWHM_2nd).^2);
RPM = (60)*(lambda*1E-9*f_Doppler/(2*n_ext*cosd(theta)*radius*1E-2));
C = Amp_Doppler/(2*Amp_2nd);

figure('Name','Fitted PSD')
plot((0:dF:F_Max),PSD,'*b')
hold on
plot((0:dF:F_Max),Fit_PSD,'-r','lineWidth',2.5)
xlabel Freq(Hz)
ylabel PSD(dB)
legend('PSD data','2 Gaussian + 2 exp','Location','best')
end